/* Copyright (C) 2000-2015 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#include "udm_config.h"
#include <stdio.h>
#include "udm_common.h" /* For assert() */
#include "udm_uniconv.h"


UDM_API(void)
UdmConvInit(UDM_CONV *cnv, UDM_CHARSET *from, UDM_CHARSET *to)
{
  cnv->from= from;
  cnv->to= to;
}


size_t
UdmConvSizeNeeded(const UDM_CONV *cnv, size_t srclen, int flags)
{
  return srclen * 14; /* TOODO34 */
}


UDM_API(int)
UdmConv(UDM_CONV *c,
        char *d, size_t dlen,
        const char *s, size_t slen,
        int flags)
{
  udm_wc_t   wc;
  int        res; /* is 16 is enough? */
  char	     *d_o= d;
  const char *s_e= s+slen;
  char	     *d_e= d+dlen;
  int        (*mb_wc)(udm_mbstate_t *mbstate,
                      const struct udm_cset_st *cs_arg, udm_wc_t *wc_arg,
                      const unsigned char *s_arg,
                      const unsigned char *e_arg, int flags);
  int        (*wc_mb)(udm_mbstate_t *mbstate,
                      const struct udm_cset_st *cs, udm_wc_t wc_arg,
                      unsigned char *s_arg,
                      unsigned char *e_arg, int flags);
  udm_mbstate_t istate= 0;
  udm_mbstate_t ostate= 0;
/*
if (slen && s[0] != 0)
  fprintf(stderr, "> %s %s %d '%.*s'\n", c->from->name, c->to->name, slen, slen, s);
*/
  mb_wc= c->from->cset->mb_wc;
  wc_mb= c->to->cset->wc_mb;
  
  while(s <s_e && d <d_e)
  {
    res= mb_wc(&istate, c->from, &wc, (const unsigned char*) s,
                                      (const unsigned char*) s_e, flags);
    if (res > 0)
    {
      s+=res;
    }
    else if ((res == UDM_CHARSET_ILSEQ)  || 
             (res == UDM_CHARSET_ILSEQ2) ||
             (res == UDM_CHARSET_ILSEQ3) ||
             (res == UDM_CHARSET_ILSEQ4) ||
             (res == UDM_CHARSET_ILSEQ5) ||
             (res == UDM_CHARSET_ILSEQ6))
    {
      /* Move at least one character */
      s+= ((res == UDM_CHARSET_ILSEQ) ? 1 : - res);
      wc= '?';
      res= wc_mb(&ostate, c->to, wc, (unsigned char*) d,
                                     (unsigned char*) d_e, flags);
      if (res <= 0)
        goto outaway;
    }
    else if (res != UDM_CHARSET_CACHEDUNI)
      goto outaway;

    res= wc_mb(&istate, c->to, wc, (unsigned char*) d,
                                   (unsigned char*) d_e, flags);
    if (res > 0)
    {
      d+= res;
    }
    else if (res == UDM_CHARSET_ILUNI)
    {
      if (flags & (UDM_RECODE_HTML_NONASCII|UDM_RECODE_HTML_NONASCII_HEX))
      {
        if ( d_e - d > 8)
        {
          if (flags & UDM_RECODE_HTML_OUT_SPECIAL)
          {
            switch (wc)
            {
            case '&': d+= sprintf(d, "&amp;"); continue;
            case '"': d+= sprintf(d, "&quot;"); continue;
            case '<': d+= sprintf(d, "&lt;");  continue;
            case '>': d+= sprintf(d, "&gt;");  continue;
            }
          }
          if (flags & UDM_RECODE_HTML_NONASCII_HEX)
            res= sprintf(d, "&#x%X;", wc);
          else
            res= sprintf(d, "&#%d;", wc);
          d+= res;
        }
        else
          break;
      }
      else
      {
        wc= '?';
        res= wc_mb(&ostate, c->to, wc, (unsigned char*) d,
                                       (unsigned char*) d_e, flags);
        if (res <= 0)
         goto outaway;
      }
    }
    else
      goto outaway;      
  }
  
outaway:  

/*
if (d - d_o  && d_o[0] != 0)
  fprintf(stderr, "< %s %s %d '%.*s'\n", c->from->name, c->to->name, d - d_o, d - d_o, d_o);
*/
  return d - d_o;
}


size_t
UdmConvLCase(UDM_UNIDATA *unidata,
             UDM_CONV *cnv, int cnvflags,
             char *dst, size_t dstlen,
             const char *src, size_t srclen)
{
#if 0
  {
    size_t len= UDM_MIN(srclen, dstlen);
    memcpy(dst, src, len);
    cs->cset->lcase(unidata, cs, dst, len);
    return len;
  }
#else
  {
    size_t conv_length;
    size_t simple_len= 0;

    size_t len= UDM_MIN(srclen, dstlen);
    for ( ; simple_len < len; simple_len++)
    {
      if (*src >= 'A' && *src <= 'Z' && *src != 'I')
        *dst++= *src++ - 'A' + 'a';
      else if (*src >= 'a' && *src <= 'z')
        *dst++= *src++;
      else if (*src >= '0' && *src <= '9')
        *dst++= *src++;
      else
        break;
    }
    dstlen-= simple_len;
    srclen-= simple_len;

    if (srclen && dstlen)
      conv_length= UdmStrToLowerExt(unidata, cnv, dst, dstlen, src, srclen, cnvflags);
    else
      conv_length= 0;
    return simple_len + conv_length;
  }
#endif
}



UDM_API(const char *)
UdmCsGroup(UDM_CHARSET *cs)
{
  switch(cs->family)
  {
    case UDM_CHARSET_ARABIC             : return "Arabic";
    case UDM_CHARSET_ARMENIAN           : return "Armenian";
    case UDM_CHARSET_BALTIC             : return "Baltic";
    case UDM_CHARSET_CELTIC             : return "Celtic";
    case UDM_CHARSET_CENTRAL            : return "Central Eur";
    case UDM_CHARSET_CHINESE_SIMPLIFIED : return "Chinese Simplified";
    case UDM_CHARSET_CHINESE_TRADITIONAL: return "Chinese Traditional";
    case UDM_CHARSET_CYRILLIC           : return "Cyrillic";
    case UDM_CHARSET_GREEK              : return "Greek";
    case UDM_CHARSET_HEBREW             : return "Hebrew";
    case UDM_CHARSET_ICELANDIC          : return "Icelandic";
    case UDM_CHARSET_JAPANESE           : return "Japanese";
    case UDM_CHARSET_KOREAN             : return "Korean";
    case UDM_CHARSET_NORDIC             : return "Nordic";
    case UDM_CHARSET_SOUTHERN           : return "South Eur";
    case UDM_CHARSET_THAI               : return "Thai";
    case UDM_CHARSET_TURKISH            : return "Turkish";
    case UDM_CHARSET_UNICODE            : return "Unicode";
    case UDM_CHARSET_VIETNAMESE         : return "Vietnamese";
    case UDM_CHARSET_WESTERN            : return "Western";
    case UDM_CHARSET_GEORGIAN           : return "Georgian";
    case UDM_CHARSET_INDIAN             : return "Indian";
    default                             : return "Unknown";
  }
}
