/* Copyright (C) 2000-2015 Lavtech.com corp. All rights reserved.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
*/

#include "udm_config.h"
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "udm_uniconv.h"
#include "udm_unicode.h"
#include "udm_unidata.h"

/* Calculates UNICODE string length */

UDM_API(size_t)
UdmUniLen(const int * u)
{
  size_t ulen= 0;
  while (*u++)
    ulen++;
  return ulen;
}

/* Allocates a copy of unicode string */

int * UdmUniDup(const int * s)
{
  int * res;
  size_t size;
  
  size=(UdmUniLen(s)+1)*sizeof(*s);
  if((res=(int*)UdmMalloc(size))==NULL)
    return(NULL);
  memcpy(res,s,size);
  return(res);
}

int * UdmUniNDup(const int *s, size_t len)
{
  int *res;
  size_t size = UdmUniLen(s);
  if (size > len) size = len;
  if((res = (int*)UdmMalloc((size + 1) * sizeof(*s))) == NULL) return(NULL);
  memcpy(res, s, size * sizeof(*s));
  res[size] = 0;
  return res;
}


/* Compare unicode strings */

int UdmUniStrCmp(const int * s1, const int * s2)
{
  while (*s1 == *s2++)
    if (*s1++ == 0)
      return (0);
  return (*s1 - *(s2 - 1));
}


/* backward unicode string compaire */
int UdmUniStrBCmp(const int *s1, const int *s2)
{
  ssize_t l1 = UdmUniLen(s1)-1, l2 = UdmUniLen(s2)-1;
  while (l1 >= 0 && l2 >= 0)
  {
    if (s1[l1] < s2[l2]) return -1;
    if (s1[l1] > s2[l2]) return 1;
    l1--;
    l2--;
  }
  if (l1 < l2) return -1;
  if (l1 > l2) return 1;
/*  if (*s1 < *s2) return -1;
  if (*s1 > *s2) return 1;*/
  return 0;
}

int UdmUniStrBNCmp(const int *s1, const int *s2, size_t count)
{ 
  ssize_t l1 = UdmUniLen(s1) - 1, l2 = UdmUniLen(s2) - 1, l = count;
  while (l1 >= 0 && l2 >= 0 && l > 0)
  {
    if (s1[l1] < s2[l2]) return -1;
    if (s1[l1] > s2[l2]) return 1;
    l1--;
    l2--;
    l--;
  }
  if (l == 0) return 0;
  if (l1 < l2) return -1;
  if (l1 > l2) return 1;
  if (*s1 < *s2) return -1;
  if (*s1 > *s2) return 1;
  return 0;
}

/* string copy */
int *UdmUniStrCpy(int *dst, const int *src)
{
  int *d = dst;
  const int *s = src;
  while (*s) {
    *d = *s; d++; s++;
  }
  *d = *s;
  return dst;
}

int *UdmUniStrNCpy(int *dst, const int *src, size_t len)
{
  int *d = dst;
  const int *s = src;
  size_t l = len;

  while (*s && l)
  {
    *d = *s; d++; s++;
    l--;
  }
  if (l) *d = *s;
  return dst;
}

/* string append */
int *UdmUniStrCat(int *s, const int *append)
{
  size_t len= UdmUniLen(s);
  UdmUniStrCpy(&s[len], append);
  return s;
}


/*
 Space characters:
 
 0009 = 0000.0000.0000.1001
 000A = 0000.0000.0000.1010
 000D = 0000.0000.0000.1101
 0020 = 0000.0000.0010.0000
 00A0 = 0000.0000.1010.0000
 
 2000 = 0010.0000.0000.0000
 2001 = 0010.0000.0000.0001
 2002 = 0010.0000.0000.0010
 2003 = 0010.0000.0000.0011
 2004 = 0010.0000.0000.0100
 2005 = 0010.0000.0000.0101
 2006 = 0010.0000.0000.0110
 2007 = 0010.0000.0000.0111
 2008 = 0010.0000.0000.1000
 2009 = 0010.0000.0000.1001
 200A = 0010.0000.0000.1010
 200B = 0010.0000.0000.1011
 202F = 0010.0000.0010.1111
 3000 = 0011.0000.0000.0000
 ----   -------------------
 CF50 - 11??.1111.?1?1.???? - not space characters

  if (X & 0xCF50)
  {
    then X is not a space character
  }

  Strip all leading and trailing space-alike characters,
  as well as all middle double space-alike characters.
  dst is not terminated with 0.

  "dst" can be eqial to "src".
  "dst" must have at least "ul" characters.

  Returns number of Unicode characters copied to "dst".
*/
size_t
UdmUniStrStripSpaces(int *dst, const int *src, size_t srclen)
{
  int *dst0= dst;
  const int *srcend= src + srclen;
  int prev_is_letter;

  for (prev_is_letter= 0; src < srcend; src++)
  {
    if ((*src & 0xCF50) || !UdmUniIsSpace(*src))
    {
      *dst++= *src;
      prev_is_letter= 1;
    }
    else if (prev_is_letter)
    {
      *dst++= 0x20;
      prev_is_letter= 0;
    }
  }
  /* Remove trailing spaces if any */
  for ( ; dst0 < dst; dst--)
  {
    if (!UdmUniIsSpace(dst[-1]))
      break;
  }
  return dst - dst0;
}
